@@ -1,14 +1,22 @@
+use ll::reporters::Level;
 use ll::Task;
 use std::sync::Arc;
 
+const FAIL_SOME: bool = false;
+
 #[tokio::main]
 async fn main() {
-    ll::add_reporter(Arc::new(ll::reporters::StdoutReporter::new()));
-    let root_task = Task::create_new("root #nostatus");
+    let mut reporter = ll::reporters::StdioReporter::new();
+    reporter.log_task_start = true;
+    reporter.timestamp_format = Some(ll::reporters::text::TimestampFormat::Local);
+    reporter.max_log_level = Level::L1;
+    ll::add_reporter(Arc::new(reporter));
+    let root_task = Task::create_new("root #nostatus #l0");
     ll::reporters::term_status::show().await;
+    ll::task_tree::TASK_TREE.set_force_flush(true);
 
     root_task
-        .spawn("will_finish_fast", |task| async move {
+        .spawn("will_finish_fast #l3", |task| async move {
             tokio::spawn(async move {
                 tokio::time::sleep(tokio::time::Duration::from_millis(6000)).await;
                 task.spawn("will_spawn_after_parent_is_done", |_| async move {
@@ -32,8 +40,11 @@ async fn main() {
 
             tokio::spawn(async move {
                 t_clone
-                    .spawn("detached_async_task", |_| async move {
-                        tokio::time::sleep(tokio::time::Duration::from_millis(8400)).await;
+                    .spawn("detached_async_task", |task| async move {
+                        for i in 0..=1000 {
+                            task.progress(i, 1000);
+                            tokio::time::sleep(tokio::time::Duration::from_millis(8)).await;
+                        }
                         Ok(())
                     })
                     .await
@@ -41,7 +52,7 @@ async fn main() {
             });
 
             let (a, b) = tokio::join!(
-                task.spawn("task_2", |task| async move {
+                task.spawn("task_2 #l3", |task| async move {
                     task.data("hey", 1);
                     task.data("yo", "sup");
                     task.data("dontprint #dontprint", 4);
@@ -53,6 +64,13 @@ async fn main() {
                     Ok(())
                 }),
                 task.spawn("task_3", |task| async move {
+                    for i in 10..=99 {
+                        let mut result = vec![];
+                        let s = format!("{} {} <<<>>>", i, "-".repeat(i % 10));
+                        result.push(s);
+                        // println!("{}", result.join("\n"));
+                    }
+
                     tokio::time::sleep(tokio::time::Duration::from_millis(2750)).await;
                     task.data_transitive("transitive", 555);
 
@@ -68,7 +86,9 @@ async fn main() {
                                 .await
                             });
 
-                            anyhow::bail!("omg no i failed");
+                            if FAIL_SOME {
+                                anyhow::bail!("omg no i failed");
+                            }
                             #[allow(unreachable_code)]
                             Ok(())
                         })
@@ -89,7 +109,7 @@ async fn main() {
             Ok(())
         })
         .await
-        .map_err(|e| ll::println!("{:?}", e))
+        .map_err(|e| println!("{:?}", e))
         .ok();
 
     drop(root_task);
