@@ -1,9 +1,10 @@
+use super::Level;
 use crate::task_tree::{TaskInternal, TaskResult, TaskStatus, TaskTree, TASK_TREE};
 use crate::uniq_id::UniqID;
 use anyhow::{Context, Result};
 use colored::Colorize;
-use crossterm::{cursor, style, terminal, ExecutableCommand};
-use std::io::stdout;
+use crossterm::{cursor, style, terminal};
+use std::io::Write;
 use std::sync::Arc;
 use std::sync::RwLock;
 
@@ -25,25 +26,56 @@ pub async fn hide() {
 pub struct TermStatus(Arc<RwLock<TermStatusInternal>>);
 
 impl TermStatus {
-    fn new(task_tree: TaskTree) -> Self {
+    fn new(task_tree: Arc<TaskTree>) -> Self {
         Self(Arc::new(RwLock::new(TermStatusInternal::new(task_tree))))
     }
 
     pub fn show(&self) {
-        self.0.write().unwrap().enabled = true;
+        let mut lock = self.0.write().unwrap();
+        if lock.enabled {
+            return;
+        } else {
+            lock.enabled = true;
+        }
+        drop(lock);
 
         let t = self.clone();
-        tokio::spawn(async move {
+        std::thread::spawn(move || {
             loop {
-                tokio::time::sleep(tokio::time::Duration::from_millis(90)).await;
-                let _lock = stdio::LOCK.lock().expect("poisoned lock");
+                // This is dumb, but it lets regular `println!` macros and such
+                // time to acquire a global mutex to print whatever they want to
+                // print. Without it this fuction will release and acquire the
+                // lock right away without letting anything print at all.
+                std::thread::sleep(std::time::Duration::from_millis(1));
+                let stdout = std::io::stdout();
+                let stderr = std::io::stderr();
+
+                // Get both locks for stdout and stderr so nothing can print to
+                // it while the status tree is displayed. If something prints
+                // while the tree si there everything will get messed up, output
+                // will be lost and parts of tree will end up as random noise.
+                let stdout_lock = stdout.lock();
+                let mut stderr_lock = stderr.lock();
+
                 let mut internal = t.0.write().unwrap();
                 if internal.enabled {
-                    internal.clear().ok();
-                    internal.print().ok();
+                    internal.print(&mut stderr_lock).ok();
                 } else {
                     break;
                 }
+                // STDIO is locked the whole time.
+                // WARN: If there a heavy IO
+                // happening this will obviously slow things down quite a bit.
+                std::thread::sleep(std::time::Duration::from_millis(50));
+
+                if internal.enabled {
+                    internal.clear(&mut stderr_lock).ok();
+                } else {
+                    break;
+                }
+
+                drop(stdout_lock);
+                drop(stderr_lock);
             }
         });
     }
@@ -51,11 +83,6 @@ impl TermStatus {
     pub fn hide(&self) {
         self.0.write().unwrap().enabled = false;
     }
-
-    pub async fn clean(&self) -> Result<()> {
-        self.0.read().unwrap().clear()?;
-        Ok(())
-    }
 }
 
 /*
@@ -74,20 +101,22 @@ type Depth = Vec<bool>;
 #[derive(Clone)]
 pub struct TermStatusInternal {
     current_height: usize,
-    task_tree: TaskTree,
+    task_tree: Arc<TaskTree>,
+    pub max_log_level: Level,
     enabled: bool,
 }
 
 impl TermStatusInternal {
-    fn new(task_tree: TaskTree) -> Self {
+    fn new(task_tree: Arc<TaskTree>) -> Self {
         Self {
             current_height: 0,
             task_tree,
+            max_log_level: Level::default(),
             enabled: false,
         }
     }
 
-    fn print(&mut self) -> Result<()> {
+    fn print(&mut self, stdio: &mut impl Write) -> Result<()> {
         let rows = self.make_status_rows()?;
         let height = rows.len();
 
@@ -95,20 +124,17 @@ impl TermStatusInternal {
             return Ok(());
         }
 
-        let stdout = stdout();
-        let mut lock = stdout.lock();
-
         self.current_height = height;
 
-        lock.execute(style::Print("\n"))?;
-        lock.execute(style::Print(rows.join("\n")))?;
-        lock.execute(style::Print("\n"))?;
+        crossterm::execute!(stdio, style::Print("\n")).ok();
+        crossterm::execute!(stdio, style::Print(rows.join("\n"))).ok();
+        crossterm::execute!(stdio, style::Print("\n")).ok();
 
         Ok(())
     }
 
     fn make_status_rows(&self) -> Result<Vec<String>> {
-        let tree = self.task_tree.0.read().unwrap();
+        let tree = self.task_tree.tree_internal.read().unwrap();
         let child_to_parents = tree.child_to_parents();
         let parent_to_children = tree.parent_to_children();
 
@@ -123,17 +149,14 @@ impl TermStatusInternal {
         while let Some((id, depth)) = stack.pop() {
             let task = tree.get_task(id).context("must be present")?;
 
-            let dontprint = task.tags.contains(NOSTATUS_TAG);
+            let dontprint = !self.should_print(task);
 
             let children_iter = parent_to_children.get(&id).into_iter().flatten().peekable();
             let mut append_to_stack = vec![];
 
             let last_visible_child = children_iter
                 .clone()
-                .filter(|id| {
-                    tree.get_task(**id)
-                        .map_or(false, |t| !t.tags.contains(NOSTATUS_TAG))
-                })
+                .filter(|id| tree.get_task(**id).map_or(false, |t| self.should_print(t)))
                 .last();
 
             // we still need to DFS the ones that we don't print to make sure
@@ -161,6 +184,11 @@ impl TermStatusInternal {
         Ok(rows)
     }
 
+    fn should_print(&self, task: &TaskInternal) -> bool {
+        let level = super::utils::parse_level(task);
+        !task.tags.contains(NOSTATUS_TAG) && (level <= self.max_log_level)
+    }
+
     fn task_row(&self, task_internal: &TaskInternal, mut depth: Depth) -> Result<String> {
         /*
 
@@ -200,6 +228,8 @@ impl TermStatusInternal {
             TaskStatus::Finished(TaskResult::Failure(_), _) => " x ".white().on_red(),
         };
 
+        let progress = make_progress(task_internal);
+
         let duration = match task_internal.status {
             TaskStatus::Finished(_, finished_at) => {
                 finished_at.duration_since(task_internal.started_at)
@@ -207,20 +237,21 @@ impl TermStatusInternal {
             _ => task_internal.started_at.elapsed(),
         }?;
 
-        let row_desc = format!("{}{} {}", indent, status, task_internal.name);
         let secs = duration.as_secs();
         let millis = (duration.as_millis() % 1000) / 100;
-        let ts = format!("{}.{}s", secs, millis);
+        let ts = format!(" [{}.{}s] ", secs, millis).dimmed();
 
-        Ok(format!("{:<140}{:>8}", row_desc, ts))
+        Ok(format!(
+            "{}{}{}{}{}",
+            indent, status, ts, progress, task_internal.name
+        ))
     }
 
-    fn clear(&self) -> Result<()> {
+    fn clear(&self, stdio: &mut impl Write) -> Result<()> {
         if self.current_height != 0 {
-            let mut stdout = stdout();
             for _ in 0..(self.current_height + 1) {
-                stdout.execute(terminal::Clear(terminal::ClearType::CurrentLine))?;
-                stdout.execute(cursor::MoveUp(1))?;
+                crossterm::execute!(stdio, terminal::Clear(terminal::ClearType::CurrentLine)).ok();
+                crossterm::execute!(stdio, cursor::MoveUp(1)).ok();
             }
         }
 
@@ -228,51 +259,17 @@ impl TermStatusInternal {
     }
 }
 
-pub mod stdio {
-    use super::TERM_STATUS;
-    use std::io::Write;
-    use std::sync::Mutex;
+fn make_progress(task: &TaskInternal) -> String {
+    const PROGRESS_BAR_LEN: i64 = 30;
 
-    lazy_static::lazy_static! {
-        pub(crate) static ref LOCK: Mutex<()> = Mutex::new(());
-        pub static ref STDOUT: BufferedStdout = BufferedStdout {};
-    }
-
-    pub fn stdout() -> BufferedStdout {
-        STDOUT.clone()
-    }
-
-    #[macro_export]
-    macro_rules! println {
-        ($($t:expr),+) => {{
-            use std::io::Write;
-            let mut stdout = $crate::reporters::term_status::stdio::stdout();
-            write!(stdout, $($t),+).unwrap();
-            write!(stdout, "\n").unwrap();
-        }};
-    }
-
-    #[derive(Clone)]
-    pub struct BufferedStdout {}
-
-    impl Write for BufferedStdout {
-        fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
-            let _lock = LOCK.lock().expect("poisoned lock");
-            let mut term_status = TERM_STATUS.0.write().unwrap();
-            if term_status.enabled {
-                term_status.clear().unwrap();
-            }
-            let mut stdout = std::io::stdout();
-            let result = stdout.write(buf);
-            if term_status.enabled {
-                term_status.print().unwrap();
-            }
-            result
-        }
-
-        fn flush(&mut self) -> std::io::Result<()> {
-            let mut stdout = std::io::stdout();
-            stdout.flush()
-        }
+    if let Some((done, total)) = &task.progress {
+        let pct_done = (done * 100) / total;
+        let done_blocks_len = std::cmp::min((PROGRESS_BAR_LEN * pct_done) / 100, PROGRESS_BAR_LEN);
+        let todo_blocks_len = PROGRESS_BAR_LEN - done_blocks_len;
+        let done_blocks = " ".repeat(done_blocks_len as usize).on_bright_green();
+        let todo_blocks = ".".repeat(todo_blocks_len as usize).on_black();
+        format!(" [{}{}] {}/{} ", done_blocks, todo_blocks, done, total)
+    } else {
+        String::new()
     }
 }
